import { BitSet } from "../hashgraph/bitset.js";
import type { Hash } from "../hashgraph/index.js";
import { type DRPPublicCredential } from "../index.js";
import type { AggregatedAttestation, Attestation } from "../proto/drp/object/v1/object_pb.js";
export interface FinalityConfig {
    finality_threshold?: number;
}
export declare class FinalityState {
    data: string;
    signerCredentials: DRPPublicCredential[];
    signerIndices: Map<string, number>;
    aggregation_bits: BitSet;
    signature?: Uint8Array;
    numberOfSignatures: number;
    constructor(hash: Hash, signers: Map<string, DRPPublicCredential>);
    addSignature(peerId: string, signature: Uint8Array, verify?: boolean): void;
    merge(attestation: AggregatedAttestation): void;
}
export declare class FinalityStore {
    states: Map<string, FinalityState>;
    finalityThreshold: number;
    constructor(config?: FinalityConfig);
    initializeState(hash: Hash, signers: Map<string, DRPPublicCredential>): void;
    getQuorum(hash: Hash): number | undefined;
    getNumberOfSignatures(hash: Hash): number | undefined;
    isFinalized(hash: Hash): boolean | undefined;
    canSign(peerId: string, hash: Hash): boolean | undefined;
    signed(peerId: string, hash: Hash): boolean | undefined;
    addSignatures(peerId: string, attestations: Attestation[], verify?: boolean): void;
    getAttestation(hash: Hash): AggregatedAttestation | undefined;
    mergeSignatures(attestations: AggregatedAttestation[]): void;
}
//# sourceMappingURL=index.d.ts.map