import { BitSet } from "./bitset.js";
import type { Vertex_Operation as Operation, Vertex } from "../proto/drp/object/v1/object_pb.js";
import { ObjectSet } from "../utils/objectSet.js";
export type { Vertex, Operation };
export type Hash = string;
export declare enum OperationType {
    NOP = "-1"
}
export declare enum ActionType {
    DropLeft = 0,
    DropRight = 1,
    Nop = 2,
    Swap = 3,
    Drop = 4
}
export declare enum SemanticsType {
    pair = 0,
    multiple = 1
}
export type ResolveConflictsType = {
    action: ActionType;
    vertices?: Hash[];
};
export type VertexDistance = {
    distance: number;
    closestDependency?: Hash;
};
export declare class HashGraph {
    peerId: string;
    resolveConflictsDRP?: (vertices: Vertex[]) => ResolveConflictsType;
    resolveConflictsACL: (vertices: Vertex[]) => ResolveConflictsType;
    semanticsTypeDRP?: SemanticsType;
    vertices: Map<Hash, Vertex>;
    frontier: Hash[];
    forwardEdges: Map<Hash, Hash[]>;
    static readonly rootHash: Hash;
    private arePredecessorsFresh;
    private reachablePredecessors;
    private topoSortedIndex;
    private vertexDistances;
    private currentBitsetSize;
    constructor(peerId: string, resolveConflictsACL: (vertices: Vertex[]) => ResolveConflictsType, resolveConflictsDRP?: (vertices: Vertex[]) => ResolveConflictsType, semanticsTypeDRP?: SemanticsType);
    resolveConflicts(vertices: Vertex[]): ResolveConflictsType;
    addToFrontier(operation: Operation): Vertex;
    addVertex(operation: Operation, deps: Hash[], peerId: string, timestamp: number, signature: Uint8Array): Hash;
    kahnsAlgorithm(origin: Hash, subgraph: ObjectSet<Hash>): Hash[];
    topologicalSort(updateBitsets?: boolean, origin?: Hash, subgraph?: ObjectSet<Hash>): Hash[];
    linearizeOperations(origin?: Hash, subgraph?: ObjectSet<string>): Operation[];
    lowestCommonAncestorMultipleVertices(hashes: Hash[], visited: ObjectSet<Hash>): Hash;
    private lowestCommonAncestorPairVertices;
    areCausallyRelatedUsingBitsets(hash1: Hash, hash2: Hash): boolean;
    swapReachablePredecessors(hash1: Hash, hash2: Hash): void;
    private _areCausallyRelatedUsingBFS;
    selfCheckConstraints(): boolean;
    areCausallyRelatedUsingBFS(hash1: Hash, hash2: Hash): boolean;
    getFrontier(): Hash[];
    getDependencies(vertexHash: Hash): Hash[];
    getVertex(hash: Hash): Vertex | undefined;
    getAllVertices(): Vertex[];
    getReachablePredecessors(hash: Hash): BitSet | undefined;
    getCurrentBitsetSize(): number;
}
//# sourceMappingURL=index.d.ts.map