export class Heap {
    static push(h, item) {
        h.push(item);
        Heap.siftUp(h, h.length() - 1);
    }
    static pop(h) {
        const n = h.length() - 1;
        if (n < 0)
            return undefined;
        Heap.swap(h, 0, n);
        Heap.siftDown(h, 0, n);
        return h.pop();
    }
    static remove(h, i) {
        const n = h.length() - 1;
        if (n !== i) {
            Heap.swap(h, i, n);
            Heap.siftDown(h, i, n);
            Heap.siftUp(h, i);
        }
        return h.pop();
    }
    static siftUp(h, i) {
        while (i > 0) {
            const parent = Math.floor((i - 1) / 2);
            if (!h.less(i, parent))
                break;
            Heap.swap(h, i, parent);
            i = parent;
        }
    }
    static siftDown(h, i, n) {
        while (true) {
            const left = 2 * i + 1;
            const right = 2 * i + 2;
            let smallest = i;
            if (left < n && h.less(left, smallest))
                smallest = left;
            if (right < n && h.less(right, smallest))
                smallest = right;
            if (smallest === i)
                break;
            Heap.swap(h, i, smallest);
            i = smallest;
        }
    }
    static swap(h, i, j) {
        h.swap(i, j);
    }
}
